'From Squeak2.8 of 18 June 2000 [latest update: #2348] on 12 July 2000 at 10:22:50 am'!
"Change Set:		dnd-fixes
Date:			12 July 2000
Author:			Michael Rueger

This change set adds new methods and fixes a few bugs in dnd:
- morphs are now informed on which item drags are requested or performed
- a nil drag source is now checked for in the animation
- drag a selector in the browser replaced the symbol with a string in the organization"!


!Object methodsFor: 'drag and drop'!
wantsDroppedMorph: aMorph event: anEvent inMorph: destinationLM onItem: itemMorph
	^self wantsDroppedMorph: aMorph event: anEvent inMorph: destinationLM! !


!EventHandler methodsFor: 'events' stamp: 'mir 6/28/2000 17:09'!
mouseDown: event fromMorph: sourceMorph 
	"Take double-clicks into account."
	(((self handlesDoubleClick: event)
		or: [self handlesStartDrag: event])
		and: [event redButtonPressed])
		ifTrue: 
			[event hand waitForClicksOrDrag: sourceMorph event: event.
			^ nil]
		ifFalse: [^ self click: event fromMorph: sourceMorph]! !


!Morph methodsFor: 'dropping/grabbing'!
wantsDroppedMorph: aMorph event: evt onItem: aItemMorph
	"Return true if the receiver wishes to accept the given morph, which is being dropped by a hand in response to the given event. The default implementation returns false.
NOTE: the event is assumed to be in global (world) coordinates."

	^self wantsDroppedMorph: aMorph event: evt ! !


!PluggableListMorph methodsFor: 'events'!
mouseEnterDragging: anEvent onItem: aMorph 
	anEvent hand hasSubmorphs ifFalse: ["no d&d"
		^ self].
	(self wantsDroppedMorph: anEvent hand firstSubmorph event: anEvent onItem: aMorph)
		ifTrue: 
			[self potentialDropMorph: aMorph]! !

!PluggableListMorph methodsFor: 'events'!
mouseLeaveDragging: anEvent onItem: aMorph 
	anEvent hand hasSubmorphs ifFalse: ["no d&d"
		^ self].
	(self wantsDroppedMorph: anEvent hand firstSubmorph event: anEvent onItem: aMorph)
		ifTrue: 
			[self resetPotentialDropMorph]! !

!PluggableListMorph methodsFor: 'events'!
mouseMove: evt
	evt yellowButtonPressed  "First check for option (menu) click"
		ifFalse: [self setSelectedMorph: nil.]
	! !

!PluggableListMorph methodsFor: 'selection'!
mouseUp: event onItem: aMorph 
	aMorph ifNotNil: [aMorph highlightForMouseDown: false].
	model okToChange ifFalse: [
		^ self].
	(aMorph containsPoint: event cursorPoint) ifFalse: [^ self].
	"No change if model is locked"
	((autoDeselect == nil or: [autoDeselect]) and: [aMorph == selectedMorph])
		ifTrue: [self setSelectedMorph: nil]
		ifFalse: [self setSelectedMorph: aMorph].
	Cursor normal show.
! !

!PluggableListMorph methodsFor: 'drag and drop'!
startDrag: evt onItem: itemMorph 
	| ddm |
	evt hand hasSubmorphs
		ifTrue: [^self].
	itemMorph ifNotNil: [itemMorph highlightForMouseDown: false].
	model okToChange ifFalse: [
		Cursor normal show.
		^ self].  "No change if model is locked"
	itemMorph ~= selectedMorph ifTrue: [self setSelectedMorph: itemMorph].
	ddm _ TransferMorph withPassenger: (self model dragPassengerFor: itemMorph inMorph: self) from: self.
	ddm dragTransferType: (self model dragTransferTypeForMorph: self).
	Preferences dragNDropWithAnimation
			ifTrue: [self model dragAnimationFor: itemMorph transferMorph: ddm].
	evt hand grabMorph: ddm! !

!PluggableListMorph methodsFor: 'drag and drop'!
wantsDroppedMorph: aMorph event: anEvent
	^ self wantsDroppedMorph: aMorph event: anEvent onItem: self potentialDropMorph! !

!PluggableListMorph methodsFor: 'drag and drop'!
wantsDroppedMorph: aMorph event: anEvent  onItem: itemMorph
	^ self model wantsDroppedMorph: aMorph event: anEvent inMorph: self onItem: itemMorph! !


!SimpleHierarchicalListMorph methodsFor: 'events'!
expand: aMorph to: level
	| allChildren |
	aMorph toggleExpandedState.
	self installEventHandlerOn: aMorph children.
	allChildren _ OrderedCollection new: 10.
	aMorph recursiveAddTo: allChildren.
	allChildren do: [:each | 
		((each canExpand
			and: [each isExpanded not])
			and: [level > 0])
			ifTrue: [self expand: each to: level-1]].
! !

!SimpleHierarchicalListMorph methodsFor: 'events'!
mouseDown: event onItem: aMorph

	| oldState |
	(aMorph inToggleArea: event) ifTrue: [
		"self setSelectedMorph: aMorph."
		event yellowButtonPressed ifTrue: [
			oldState _ aMorph isExpanded.
			scroller submorphs copy do: [ :each |
				(each canExpand and: [each isExpanded = oldState]) ifTrue: [
					each toggleExpandedState.
					self installEventHandlerOn: each children.
				].
			].
		] ifFalse: [
			aMorph toggleExpandedState.
			self installEventHandlerOn: aMorph children.
		].
		(scroller submorphs includes: self selectedMorph) not
			ifTrue: [self selectedMorph: nil].
		self adjustSubmorphPositions.
		^self
	].
	event yellowButtonPressed ifTrue: [^ self yellowButtonActivity: event shiftPressed].
	model okToChange ifFalse: [^ self].  "No change if model is locked"
	((autoDeselect == nil or: [autoDeselect]) and: [aMorph == selectedMorph])
		ifTrue: [self setSelectedMorph: nil]
		ifFalse: [self setSelectedMorph: aMorph].
! !

!SimpleHierarchicalListMorph methodsFor: 'events'!
mouseEnterDragging: anEvent onItem: aMorph 
	anEvent hand hasSubmorphs ifFalse: ["no d&d"
		^ self].
	(self wantsDroppedMorph: anEvent hand firstSubmorph event: anEvent onItem: aMorph)
		ifTrue: 
			[self potentialDropMorph: aMorph]! !

!SimpleHierarchicalListMorph methodsFor: 'events'!
mouseLeaveDragging: anEvent onItem: aMorph 
	anEvent hand hasSubmorphs ifFalse: ["no d&d"
		^ self].
	(self wantsDroppedMorph: anEvent hand firstSubmorph event: anEvent onItem: aMorph)
		ifTrue: 
			[self resetPotentialDropMorph]! !

!SimpleHierarchicalListMorph methodsFor: 'dropping/grabbing' stamp: 'mir 6/28/2000 15:43'!
installEventHandlerOn: morphList
	| handler |
	handler _ EventHandler new.
	handler
		on: #mouseDown send: #mouseDown:onItem: to: self.
	self dragEnabled
		ifTrue: [handler
				on: #startDrag
				send: #startDrag:onItem:
				to: self].
	self dropEnabled
		ifTrue: 
			[handler
				on: #mouseEnterDragging
				send: #mouseEnterDragging:onItem:
				to: self.
			handler
				on: #mouseLeaveDragging
				send: #mouseLeaveDragging:onItem:
				to: self].

	morphList do: [:m | m eventHandler: handler].
! !

!SimpleHierarchicalListMorph methodsFor: 'dropping/grabbing'!
wantsDroppedMorph: aMorph event: anEvent
	^ self wantsDroppedMorph: aMorph event: anEvent onItem: self potentialDropMorph! !

!SimpleHierarchicalListMorph methodsFor: 'dropping/grabbing'!
wantsDroppedMorph: aMorph event: anEvent onItem: itemMorph
	^ self model wantsDroppedMorph: aMorph event: anEvent inMorph: self onItem: itemMorph! !


!TransferMorph methodsFor: 'private' stamp: 'mir 6/28/2000 17:05'!
animationForMoveSuccess: success 
	| start stop slideForm |
	self source ifNil: [^self].
	success
		ifTrue: [^ self]
		ifFalse: 
			[start _ self fullBounds origin.
			stop _ self source bounds origin].
	start = stop ifTrue: [^ self].
	slideForm _ self imageFormForRectangle: ((self fullBounds origin corner: self fullBounds corner + self activeHand 
shadowOffset)
					merge: self activeHand bounds).
	slideForm offset: 0 @ 0.
	slideForm
		slideWithFirstFrom: start
		to: stop
		nSteps: 12
		delay: 20! !

